import 'package:flutter/cupertino.dart';
import 'package:hive/hive.dart';
import 'package:osam/src/domain/persist.dart';
import 'package:path_provider/path_provider.dart';

class PersistRepository<AS, US> implements Persist<AS, US> {
  Box<AS> _appPersist;
  Box<US> _uiPersist;
  Box<int> _versionBox;
  bool _isInitialized = false;
  final String appName;

  PersistRepository({@required this.appName});

  @override
  Future<int> get version async {
    Hive.init((await getApplicationDocumentsDirectory()).path);
    _isInitialized = true;
    _versionBox = await Hive.openBox(appName + '_version_');
    return _versionBox.get('version');
  }

  @override
  Future<void> init() async {
    if (!_isInitialized)
      Hive.init((await getApplicationDocumentsDirectory()).path);
    _appPersist = await Hive.openBox(appName);
    _uiPersist = await Hive.openBox(appName + '_ui_');
    _versionBox ??= await Hive.openBox(appName + '_version_');
  }

  @override
  Future<void> save({AS state, US uiState, int version}) async {
    await _appPersist?.put('appState', state);
    await _uiPersist?.put('uiState', uiState);
    await _versionBox?.put('version', version);
  }

  @override
  AS get appState => _appPersist.get('appState');

  @override
  US get uiState => _uiPersist.get('uiState');

  @override
  Future<void> delete() async {
    await Hive.deleteBoxFromDisk(appName);
    await Hive.deleteBoxFromDisk(appName + '_ui_');
    await Hive.deleteBoxFromDisk(appName + '_version_');
    _versionBox = null;
  }

  @override
  bool get isPersistEnabled => _appPersist == null ? false : true;

  @override
  Future<void> deleteFromStorage() => Hive.deleteFromDisk();
}
