class UseCaseNotFoundException implements Exception {
  final Type useCaseType;

  UseCaseNotFoundException(this.useCaseType);
}

class UseCaseContainer {
  final List<UseCase> _useCases;
  final _useCasesMap = <Type, UseCase>{};

  U useCase<U extends UseCase>() {
    final useCase = _useCasesMap[U];
    if (useCase == null) {
      throw UseCaseNotFoundException(U);
    } else
      return useCase;
  }

  UseCaseContainer(this._useCases) {
    if (_useCases?.isNotEmpty ?? false) {
      for (final useCase in _useCases) {
        useCase._useCaseContainer = this;
        _useCasesMap[useCase.runtimeType] = useCase;
      }
    }
  }

  void updateUseCases(List<Object> newUseCases) => newUseCases
      .forEach((useCase) => _useCasesMap[useCase.runtimeType] = useCase);
}

class UseCase {
  UseCaseContainer _useCaseContainer;

  UseCaseContainer get useCaseContainer => _useCaseContainer;
}
